@@ -693,13 +693,17 @@ uint32_t
 ovn_allocate_tnlid(struct hmap *set, const char *name, uint32_t min,
                    uint32_t max, uint32_t *hint)
 {
-    for (uint32_t tnlid = next_tnlid(*hint, min, max); tnlid != *hint;
-         tnlid = next_tnlid(tnlid, min, max)) {
+    /* Normalize hint, because it can be outside of [min, max]. */
+    *hint = next_tnlid(*hint, min, max);
+
+    uint32_t tnlid = *hint;
+    do {
         if (ovn_add_tnlid(set, tnlid)) {
             *hint = tnlid;
             return tnlid;
         }
-    }
+        tnlid = next_tnlid(tnlid, min, max);
+    } while (tnlid != *hint);
 
     static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(1, 1);
     VLOG_WARN_RL(&rl, "all %s tunnel ids exhausted", name);
